/*
 * Copyright 2012 uaiHebert Solucoes em Informatica
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * */
package com.uaihebert.factory;

import com.uaihebert.model.BasicQueryElements;
import com.uaihebert.model.EasyCriteria;
import com.uaihebert.model.EasyCriteriaImp;

import javax.persistence.EntityManager;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

/**
 * @author uaiHebert.com
 *
 * Class that will create an instance of the EasyCriteria
 */
public abstract class EasyCriteriaFactory {

    /**
     * Creates a new instance of the EasyCriteria that
     * will generate a TypedQuery.
     *
     * @param entityManager Entity Manager implementation
     * @param entityClass The class to be queried
     * @return a EasyCriteria instance.
     */
    public static <T> EasyCriteria<T> createQueryCriteria(EntityManager entityManager, Class<T> entityClass) {
        CriteriaBuilder criteriaBuilder = entityManager.getCriteriaBuilder();
        CriteriaQuery<T> criteriaQuery = criteriaBuilder.createQuery(entityClass);
        Root<T> mainRoot = criteriaQuery.from(entityClass);
        criteriaQuery.select(mainRoot);

        BasicQueryElements<T> basicQueryElements = new BasicQueryElements<T>(criteriaBuilder, criteriaQuery, mainRoot, entityManager, entityClass);

        return new EasyCriteriaImp<T>(basicQueryElements);
    }
}
