/*
 * Copyright 2012 uaiHebert Solucoes em Informatica
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * */
package com.uaihebert.model;

import javax.persistence.TypedQuery;
import java.util.List;

public class EasyCriteriaImp<T> implements EasyCriteria<T> {
    private BasicQueryElements<T> basicQueryElements;
    private int firstResult;
    private int maxResults;

    public EasyCriteriaImp(BasicQueryElements<T> basicQueryElements) {
        this.basicQueryElements = basicQueryElements;
    }

    public List<T> getResultList() {
        TypedQuery<T> query = basicQueryElements.getTypedQuery();

        if (firstResult > 0) {
            query.setFirstResult(firstResult);
            firstResult = 0;
        }

        if (maxResults > 0) {
            query.setMaxResults(maxResults);
            maxResults = 0;
        }

        return query.getResultList();
    }

    public T getSingleResult() {
        TypedQuery<T> query = basicQueryElements.getTypedQuery();

        return query.getSingleResult();
    }

    public EasyCriteriaImp<T> andEquals(String attributeName, Object value) {
        basicQueryElements.andEquals(attributeName, value);
        return this;
    }

    public EasyCriteria<T> andNotEquals(String attributeName, Object value) {
        basicQueryElements.andNotEquals(attributeName, value);
        return this;
    }

    public EasyCriteria<T> orNotEquals(String attributeName, Object... values) {
        basicQueryElements.orNotEquals(attributeName, values);
        return this;
    }

    public EasyCriteria<T> andGreaterThan(String attributeName, Object value){
        basicQueryElements.andGreaterThan(attributeName, value);
        return this;
    }


    public EasyCriteria<T> andGreaterOrEqualTo(String attributeName, Object value) {
        basicQueryElements.andGreaterOrEqualTo(attributeName, value);
        return this;
    }

    public EasyCriteria<T> andLessThan(String attributeName, Object value) {
        basicQueryElements.andLessThan(attributeName, value);
        return this;
    }

    public EasyCriteria<T> andLessOrEqualTo(String attributeName, Object value) {
        basicQueryElements.andLessOrEqualTo(attributeName, value);
        return this;
    }

    public EasyCriteria<T> innerJoin(String joinName) {
        basicQueryElements.innerJoin(joinName);
        return this;
    }

    public EasyCriteria<T> setDistinctTrue() {
        basicQueryElements.setDistinctTrue();
        return this;
    }

    public EasyCriteria<T> leftJoin(String joinName) {
        basicQueryElements.leftJoin(joinName);
        return this;
    }

    public EasyCriteria<T> innerJoinFetch(String joinName) {
        basicQueryElements.innerJoinFetch(joinName);
        return this;
    }

    public EasyCriteria<T> leftJoinFetch(String joinName) {
        basicQueryElements.leftJoinFetch(joinName);
        return this;
    }

    public EasyCriteria<T> andBetween(String attributeName, Object valueA, Object valueB){
        basicQueryElements.andBetween(attributeName, valueA, valueB);
        return this;
    }

    public EasyCriteria<T> andIsNull(String attributeName) {
        basicQueryElements.andIsNull(attributeName);
        return this;
    }

    public EasyCriteria<T> andIsNotNull(String attributeName) {
        basicQueryElements.andIsNotNull(attributeName);
        return this;
    }

    public EasyCriteria<T> andCollectionIsEmpty(String collectionName) {
        basicQueryElements.andCollectionIsEmpty(collectionName);
        return this;
    }

    public EasyCriteria<T> andCollectionIsNotEmpty(String collectionName) {
        basicQueryElements.andCollectionIsNotEmpty(collectionName);
        return this;
    }

    public EasyCriteria<T> andStringLike(String attributeName, String value) {
        basicQueryElements.whereStringLike(attributeName, value);
        return this;
    }

    public EasyCriteria<T> andStringNotLike(String attributeName, String value) {
        basicQueryElements.whereStringNotLike(attributeName, value);
        return this;
    }

    public EasyCriteria<T> andStringIn(String attributeName, List<String> values) {
        basicQueryElements.andStringIn(attributeName, values);
        return this;
    }

    public EasyCriteria<T> andStringNotIn(String attributeName, List<String> values) {
        basicQueryElements.andStringNotIn(attributeName, values);
        return this;
    }

    public EasyCriteria<T> orderByAsc(String attributeName) {
        basicQueryElements.orderByAsc(attributeName);
        return this;
    }

    public EasyCriteria<T> orderByDesc(String attributeName) {
        basicQueryElements.orderByDesc(attributeName);
        return this;
    }

    public void setFirstResult(int firstResult) {
        this.firstResult = firstResult;
    }

    public void setMaxResults(int maxResults) {
        this.maxResults = maxResults;
    }

    public EasyCriteria<T> andJoinEquals(String joinName, String attributeName, Object value) {
        basicQueryElements.andJoinEquals(joinName, attributeName, value);
        return this;
    }

    public EasyCriteria<T> andJoinNotEquals(String joinName, String attributeName, Object value) {
        basicQueryElements.andJoinNotEquals(joinName, attributeName, value);
        return this;
    }

    public EasyCriteria<T> andJoinGreaterThan(String joinName, String attributeName, Object value) {
        basicQueryElements.andJoinGreaterThan(joinName, attributeName, value);
        return this;
    }

    public EasyCriteria<T> andJoinGreaterOrEqualTo(String joinName, String attributeName, Object value) {
        basicQueryElements.andJoinGreaterOrEqualTo(joinName, attributeName, value);
        return this;
    }

    
    public EasyCriteria<T> andJoinLessThan(String joinName, String attributeName, Object value) {
        basicQueryElements.andJoinLessThan(joinName, attributeName, value);
        return this;
    }

    public EasyCriteria<T> andJoinLessOrEqualTo(String joinName, String attributeName, Object value) {
        basicQueryElements.andJoinLessOrEqualTo(joinName, attributeName, value);
        return this;
    }

    public EasyCriteria<T> andJoinBetween(String joinName, String attributeName, Object valueA, Object valueB){
        basicQueryElements.andJoinBetween(joinName, attributeName, valueA, valueB);
        return this;
    }

    public EasyCriteria<T> andJoinAttributeIsNull(String joinName, String attributeName) {
        basicQueryElements.andJoinAttributeIsNull(joinName, attributeName);
        return this;
    }

    public EasyCriteria<T> andJoinAttributeIsNotNull(String joinName, String attributeName) {
        basicQueryElements.andJoinAttributeIsNotNull(joinName, attributeName);
        return this;
    }

    public EasyCriteria<T> andJoinListIsEmpty(String joinName, String listName) {
        basicQueryElements.andJoinListIsEmpty(joinName, listName);
        return this;
    }

    public EasyCriteria<T> andJoinCollectionIsEmpty(String joinName, String collectionName) {
        basicQueryElements.andJoinCollectionIsEmpty(joinName, collectionName);
        return this;
    }

    
    public EasyCriteria<T> andJoinCollectionIsNotEmpty(String joinName, String collectionName) {
        basicQueryElements.andJoinCollectionIsNotEmpty(joinName, collectionName);
        return this;
    }
    
    public EasyCriteria<T> andJoinStringLike(String joinName, String attributeName, String value) {
        basicQueryElements.andJoinStringLike(joinName, attributeName, value);
        return this;
    }

    public EasyCriteria<T> andJoinSetIsEmpty(String joinName, String setName) {
        basicQueryElements.andJoinSetIsEmpty(joinName, setName);
        return this;
    }

    public EasyCriteria<T> andJoinListIsNotEmpty(String joinName, String listName) {
        basicQueryElements.andJoinListIsNotEmpty(joinName, listName);
        return this;
    }

    public EasyCriteria<T> andJoinSetIsNotEmpty(String joinName, String setName) {
        basicQueryElements.andJoinSetIsNotEmpty(joinName, setName);
        return this;
    }

    public EasyCriteria<T> andJoinStringNotLike(String joinName, String attributeName, String value) {
        basicQueryElements.andJoinStringNotLike(joinName, attributeName, value);
        return this;
    }

    
    public EasyCriteria<T> andJoinStringIn(String joinName, String attributeName, List<String> values) {
        basicQueryElements.andJoinStringIn(joinName, attributeName, values);
        return this;
    }

    
    public EasyCriteria<T> andJoinStringNotIn(String joinName, String attributeName, List<String> values) {
        basicQueryElements.andJoinStringNotIn(joinName, attributeName, values);
        return this;
    }

    public EasyCriteria<T> orEquals(String attributeName, Object... values) {
        basicQueryElements.orEquals(attributeName, values);
        return this;
    }

    public EasyCriteria<T> orEquals(int index, String attributeName, Object... values) {
        basicQueryElements.orEquals(index, attributeName, values);
        return this;
    }

    public EasyCriteria<T> addAndSeparatedByOr(int index, String attributeName, Object value){
        basicQueryElements.addAndSeparatedByOr(index, attributeName, value);
        return this;
    }
}