package com.uaihebert.util;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.util.logging.Logger;

public abstract class ReflectionUtil {

    private static final Logger LOG = Logger.getLogger("EasyCriteria");

    private static String getAttributeType(Class<?> entityClass, String attributeName) {
        if (attributeName == null || attributeName.isEmpty()) {
            throw new IllegalArgumentException("attributeName parameter cannot be null");
        }

        try {
            BeanInfo beanInfo = Introspector.getBeanInfo(entityClass);
            PropertyDescriptor[] descriptors = beanInfo.getPropertyDescriptors();

            for (PropertyDescriptor pd : descriptors) {
                if (pd.getName().toLowerCase().equals(attributeName.toLowerCase())) {
                    return pd.getPropertyType().toString();
                }
            }
        } catch (Exception e) {
            LOG.severe("We could not find the parameter: " + attributeName);
        }

        throw new IllegalArgumentException("We could not find the parameter: " + attributeName + " in the given class: " + entityClass);
    }

    public static boolean isDouble(Class<?> entityClass, String attributeName) {
        final String attributeType = getAttributeType(entityClass, attributeName);

        return attributeType.toLowerCase().contains("double");
    }

    public static <T> boolean isFloat(Class<T> entityClass, String attributeName) {
        final String attributeType = getAttributeType(entityClass, attributeName);

        return attributeType.toLowerCase().contains("float");
    }

    public static <T> boolean isLong(Class<T> entityClass, String attributeName) {
        final String attributeType = getAttributeType(entityClass, attributeName);

        return attributeType.toLowerCase().contains("long");
    }

    public static <T> boolean isInteger(Class<T> entityClass, String attributeName) {
        final String attributeType = getAttributeType(entityClass, attributeName);

        return attributeType.toLowerCase().contains("integer") || attributeType.toLowerCase().equals("int");
    }

    public static <T> boolean isString(Class<T> entityClass, String attributeName) {
        final String attributeType = getAttributeType(entityClass, attributeName);

        return attributeType.toLowerCase().contains("string");
    }

    public static <T> boolean isDate(Class<T> entityClass, String attributeName) {
        final String attributeType = getAttributeType(entityClass, attributeName);

        return attributeType.toLowerCase().contains("util.date");
    }

    public static <T> boolean isCalendar(Class<T> entityClass, String attributeName) {
        final String attributeType = getAttributeType(entityClass, attributeName);

        return attributeType.toLowerCase().contains("calendar");
    }

    public static <T> boolean isList(Class<T> entityClass, String collectionName) {
        final String attributeType = getAttributeType(entityClass, collectionName);

        return attributeType.toLowerCase().contains("list");
    }

    public static <T> boolean isSet(Class<T> entityClass, String collectionName) {
        final String attributeType = getAttributeType(entityClass, collectionName);

        return attributeType.toLowerCase().contains("set");
    }

    public static <T> boolean isCollection(Class<T> entityClass, String collectionName) {
        final String attributeType = getAttributeType(entityClass, collectionName);

        return attributeType.toLowerCase().contains("collection");
    }

    public static <T> boolean isBigDecimal(Class<T> entityClass, String collectionName) {
        final String attributeType = getAttributeType(entityClass, collectionName);

        return attributeType.toLowerCase().contains("bigdecimal");
    }

    public static boolean isJoinAttributeInteger(String canonicalName, String attributeName) {
        try {
            Class<?> joinClass = Class.forName(canonicalName);
            return isInteger(joinClass, attributeName);
        } catch (ClassNotFoundException e) {
            System.err.println("Something really bad happened: " + e.getMessage());
            e.printStackTrace();
        }

        return false;
    }

    public static boolean isJoinAttributeLong(String canonicalName, String attributeName) {
        try {
            Class<?> joinClass = Class.forName(canonicalName);
            return isLong(joinClass, attributeName);
        } catch (ClassNotFoundException e) {
            System.err.println("Something really bad happened: " + e.getMessage());
            e.printStackTrace();
        }

        return false;
    }

    public static boolean isJoinAttributeDouble(String canonicalName, String attributeName) {
        try {
            Class<?> joinClass = Class.forName(canonicalName);
            return isDouble(joinClass, attributeName);
        } catch (ClassNotFoundException e) {
            System.err.println("Something really bad happened: " + e.getMessage());
            e.printStackTrace();
        }

        return false;
    }

    public static boolean isJoinAttributeFloat(String canonicalName, String attributeName) {
        try {
            Class<?> joinClass = Class.forName(canonicalName);
            return isFloat(joinClass, attributeName);
        } catch (ClassNotFoundException e) {
            System.err.println("Something really bad happened: " + e.getMessage());
            e.printStackTrace();
        }

        return false;
    }

    public static boolean isJoinAttributeDate(String canonicalName, String attributeName) {
        try {
            Class<?> joinClass = Class.forName(canonicalName);
            return isDate(joinClass, attributeName);
        } catch (ClassNotFoundException e) {
            System.err.println("Something really bad happened: " + e.getMessage());
            e.printStackTrace();
        }

        return false;
    }

    public static boolean isJoinAttributeCalendar(String canonicalName, String attributeName) {
        try {
            Class<?> joinClass = Class.forName(canonicalName);
            return isCalendar(joinClass, attributeName);
        } catch (ClassNotFoundException e) {
            System.err.println("Something really bad happened: " + e.getMessage());
            e.printStackTrace();
        }

        return false;
    }

    public static boolean isJoinAttributeString(String canonicalName, String attributeName) {
        try {
            Class<?> joinClass = Class.forName(canonicalName);
            return isString(joinClass, attributeName);
        } catch (ClassNotFoundException e) {
            System.err.println("Something really bad happened: " + e.getMessage());
            e.printStackTrace();
        }

        return false;
    }

    public static boolean isJoinAttributeBigDecimal(String canonicalName, String attributeName) {
        try {
            Class<?> joinClass = Class.forName(canonicalName);
            return isBigDecimal(joinClass, attributeName);
        } catch (ClassNotFoundException e) {
            System.err.println("Something really bad happened: " + e.getMessage());
            e.printStackTrace();
        }

        return false;
    }

    public static boolean isAcceptableCollection(Class<?> javaType) {
        return javaType.getCanonicalName().toLowerCase().contains("list") || javaType.getCanonicalName().toLowerCase().contains("set") || javaType.getCanonicalName().toLowerCase().contains("collection");
    }
}